#!/usr/bin/php
<?php

/* 
// Caution $_ENV variables are not be known here yet when variables_order
// does NOT contain E!!!
if ( isset($_ENV['MYENV_BASE']) ) {
  $basedir = strval($_ENV['MYENV_BASE']);
}
// We have to guess:
else {
  $basedir = dirname(dirname(__FILE__));
}
*/

// I think guessing ist better than the above stuff!!!
$basedir = dirname(dirname(__FILE__));

require_once($basedir . '/lib/myEnv.inc');
require_once($basedir . '/lib/Constants.inc');

$rc = OK;

if ( checkMyEnvRequirements() == ERR ) {
  $rc = 535;
  exit($rc);
}

// The first is possibly a bug!!!
$lHome   = $_ENV['HOME'] . '/product';
$lHome   = '/home/mysql' . '/product';

$lDebug  = isset($_ENV['MYENV_DEBUG']) ? $_ENV['MYENV_DEBUG'] : 0;
$basedir = $_ENV['MYENV_BASE'];

$lConfigurationFile = '/etc/myenv/myenv.conf';
if ( ! is_file($lConfigurationFile) ) {
	$rc = 534;
	print "Warning: Configuration file $lConfigurationFile does not exist!\n";
	print "         Please create or copy from $lConfigurationFile.template (rc=$rc).\n";
	exit($rc);
}

list($ret, $aConfiguration) = getConfiguration($lConfigurationFile);
$aDbNames = getSectionTitles($aConfiguration);

// Get available release versions

$aReleaseVersion = array();
foreach ( glob("$lHome/*", GLOB_ONLYDIR) as $dir ) {
	$v = extractVersion($dir);
	if ( $v == 'unknown' ) {
		$v = getVersionFromMysqld('/usr');
	}
	array_push($aReleaseVersion, $v);
}
$aReleaseVersion = array_unique($aReleaseVersion, SORT_REGULAR);

if ( $lDebug != '' ) {
	var_dump($aReleaseVersion);
}

// Get version of each instance
foreach ( $aConfiguration as $db => $value ) {

	if ( $db != 'default' ) {
		$v = extractVersion($aConfiguration[$db]['basedir']);
		if ( $v == 'unknown' ) {
			$v = getVersionFromMysqld('/usr');
		}
		$aConfiguration[$db]['version'] = $v;
	}
}

// Check if shell is interactive or not
// https://www.gnu.org/software/bash/manual/html_node/Is-this-Shell-Interactive_003f.html
// himBH
// hBc
// non-interactive
if ( isset($_ENV['PS1']) && ($_ENV['PS1'] != '') ) {
	// Get number of columns of screen:
	$lNumberOfColumns = trim(`tput cols`);
}
// interactive
else {
	// Get number of columns of screen:
	$lNumberOfColumns = 80;
}

if ( $lDebug != '' ) {
	print "Number of Columns: $lNumberOfColumns\n";
}


// Releases does not make sense anymore with all the differnce forks
//print "\n";
//print "Releases : " . implode(' ', $aReleaseVersion) . "\n";
print "\n";

// Display up and down instances

$aUp      = array();
$aDown    = array();
$aPassive = array();   // For passive databases on an active/passive fail-over cluster

foreach ( $aDbNames as $db ) {

  if ( count($aConfiguration[$db]) == 0 ) {
    $rc = 539;
    print "ERROR #$rc\n";
    continue;
  }

	// Ignore passive databases in an active/passive fail-over Cluster
	// Criteria is that datadir is missing!
	if ( isset($aConfiguration[$db]['ignore-passive'])
	  && ($aConfiguration[$db]['ignore-passive'] == 'yes')
		&& ( ! is_dir($aConfiguration[$db]['datadir']))
	  ) {
		$aPassive[$db] = "$db (" . $aConfiguration[$db]['version'] . ")";
	}
	// Database is NOT passive so check if it is up or down.
	else {

		$ret = checkDatabase($aConfiguration[$db]);

		if ( $ret != 0 ) {
			$aDown[$db] = "$db (" . $aConfiguration[$db]['version'] . ")";
		}
		else {
			$aUp[$db]   = "$db (" . $aConfiguration[$db]['version'] . ")";
		}
	}
}

if ( $lDebug != '' ) {
	print 'Up  : ';
	var_dump($aUp);
	print 'Down: ';
	var_dump($aDown);
	print 'Passive: ';
	var_dump($aPassive);
}

// Print Up line
$cnt = 1;
foreach ( explode("\n", wordwrap(implode(' ', $aUp), $lNumberOfColumns-12)) as $line ) {

  if ( $cnt == 1 ) {
    print "Up       : " . $line . "\n";
  }
  else {
    print "           " . $line . "\n";
  }
  $cnt++;
}
print "\n";

// Print Down line
$cnt = 1;
foreach ( explode("\n", wordwrap(implode(' ', $aDown), $lNumberOfColumns-12)) as $line ) {

  if ( $cnt == 1 ) {
    print "Down     : " . $line . "\n";
  }
  else {
    print "           " . $line . "\n";
  }
  $cnt++;
}
print "\n";

// Print Passive line only if some are there
if ( count($aPassive) > 0 ) {
	$cnt = 1;
	foreach ( explode("\n", wordwrap(implode(' ', $aPassive), $lNumberOfColumns-12)) as $line ) {

		if ( $cnt == 1 ) {
			print "Passive  : " . $line . "\n";
		}
		else {
			print "           " . $line . "\n";
		}
		$cnt++;
	}
	print "\n";
}

// Find the longes instance name
$max_len = max(array_map('strlen', $aDbNames));

// Print all instances
foreach ( $aDbNames as $db ) {

	$aSchema = getSchemaNames($aConfiguration[$db]['datadir']);
	if ( $lDebug != '' ) {
		print_r($aSchema);
	}

	// hide schema, e.g mysql, ndbinfo, test, performance_schema, pbxt
	if ( array_key_exists('hideschema', $aConfiguration[$db]) ) {

		foreach ( explode(',', $aConfiguration[$db]['hideschema']) as $tohide ) {
		
			if ( ($key = array_search($tohide, $aSchema)) !== false ) {
				unset($aSchema[$key]);
			}
		}
	}

	// Ignore passive databases in an active/passive fail-over Cluster
	// Criteria is that datadir is missing!
	if ( isset($aConfiguration[$db]['ignore-passive'])
	  && ($aConfiguration[$db]['ignore-passive'] == 'yes')
		&& ( ! is_dir($aConfiguration[$db]['datadir']))
	  ) {
	  // noop
	}
	else {
		// Split the schema output
		$cnt = 1;
		
		foreach ( explode("\n", wordwrap(implode(' ', $aSchema), $lNumberOfColumns-$max_len-13)) as $line ) {

			if ( $cnt == 1 ) {
				printf("%-" . $max_len . "s (%5d) : %s\n", $db, $aConfiguration[$db]['port'], $line);
			}
			else {
				printf("%s %s\n", str_repeat(' ', $max_len+10), $line);
			}
			$cnt++;
		}
	}
}
print "\n";


// Interface/hook  for customer scripts/plugins

debug('Plugins to execute:');
$cnt = 0;

$lMyNameBase = basename($argv[0], '.php');
foreach ( glob($basedir . '/plg/' . $lMyNameBase . '/*') as $plugin ) {

  debug(' ' . $plugin);
  if ( is_executable($plugin) !== TRUE ) {
    print "ERROR: $plugin is NOT executable.\n";
  }
  else {

    $cmd = $plugin;
    list($ret, $output, $stdout, $stderr) = my_exec($cmd);
    foreach ( $stdout as $line ) {
      print "$line\n";
    }

    if ( $ret != 0 ) {
      $rc = $ret;
    }
  }
  $cnt++;
}
if ( $cnt > 0 ) {
  print "\n";
}

checkForOldStuff();

exit($rc);
?>
