<?php

/*

  getInstanceBlackList()
  answerQuestion($str, $aAllowedKeys, $defaultKey)
  copyFromTemplate($pConfFileTemplate, $pConfFile)
  addInstance()
  deleteInstance($pInstance)
  changeInstance($pInstance)
  getInstances($aConfiguration)
  installInstance($aConfiguration)
  askForSocket($aConfiguration)
  askForMyCnf($aConfiguration)
	downloadChunkedFile($url, $outfile)

*/

// ---------------------------------------------------------------------
function getInstanceBlackList()
// ---------------------------------------------------------------------
{
	return array('mysql', 'mysqld');
}

// ---------------------------------------------------------------------
function answerQuestion($str, $aAllowedKeys, $defaultKey)
// ---------------------------------------------------------------------
{
	while ( true ) {

		// readline always caused crippled output for multi-line output
		// thus we do it our self
		// $input = strtolower(readline($str));
		print $str;
		$input = strtolower(readline(''));

		// on CR do default Key
		if ( $input == '' ) {
			$input = $defaultKey;
		}

		foreach( $aAllowedKeys as $key ) {
			if ( $key == $input ) {
				return $input;
			}
		}
	}
}

// ---------------------------------------------------------------------
function copyFromTemplate($pConfFileTemplate, $pConfFile)
// ---------------------------------------------------------------------
{
	$rc = OK;

	// Copy template
	print "Copy $pConfFileTemplate to $pConfFile\n";
	if ( ! @copy($pConfFileTemplate, $pConfFile) ) {

		$rc = 555;
		$a = error_get_last();
		$msg = "Cannot copy template $pConfFileTemplate to $pConfFile. Reason: " . $a['message'] . " (rc=$rc).";
		print "$msg\n";
		print "Aborting...\n";
		return $rc;
	}

	return $rc;
}

// ---------------------------------------------------------------------
function addInstance()
// ---------------------------------------------------------------------
{
	global $aConfiguration;

	// todo: check if default already exists!
	$default = 'mysqld1';
	print "\nYou have to enter a new instance name. An instance name can consist of 0-9, a-z, A-Z, - and _.\n";
	print "Examples: test, prod, dev, [$default] mysqld-3306\n";
	print "\n";

	$instance = readline("Enter a new instance name: ");
	if ( $instance == '' ) {
		$instance = $default;
	}

	// Check for existing instance name here!
	foreach ( $aConfiguration as $instances ) {
		if ( $instances == $instance ) {
			$rc = 552;
			error("Instance name $instance already exists (rc=$rc).\n");
			return $rc;
		}
	}

	// Check if instance name is on black list:
	$aBlackList = getInstanceBlackList();
	foreach ( $aBlackList as $name ) {
		if ( $name == $instance ) {
			$rc = 553;
			error("Instance name $instance is on blacklist (rc=$rc).");
			readline("Press Enter choose an other one...");
			return $rc;
		}
	}

	$aConfiguration[$instance]               = array();
	$aConfiguration[$instance]['start']      = 'yes';
	$aConfiguration[$instance]['stop']       = 'yes';
	$aConfiguration[$instance]['hideschema'] = 'mysql,performance_schema';
	changeInstance($instance);
}

// ---------------------------------------------------------------------
function deleteInstance($pInstance)
// ---------------------------------------------------------------------
{
	$rc = OK;
	global $aConfiguration;

	if ( ! isset($aConfiguration[$pInstance]) ) {
		$rc = 507;
		error("Instance $pInstance does not exist. Please choose a valid instance name (rc=$rc).");
		return $rc;
	}

	$ret = checkDatabase($aConfiguration[$pInstance]);

	if ( $ret != 0 ) {

		print "Delete datadir (" . $aConfiguration[$pInstance]['datadir'] . ") and basedir (" . $aConfiguration[$pInstance]['basedir'] . ") manually.\n";
		unset($aConfiguration[$pInstance]);
		print "Instance $pInstance has been deleted from MyEnv.\n";
		print "Do not forget to save configuration before leaving installer.\n";
		readline("Press Enter to continue...");
		print "\n\n";
	}
	else {
		$rc = 554;
		error("Please stop instence first before deleting it (rc=$rc).");
		return $rc;
	}

	return $rc;
}

// ---------------------------------------------------------------------
function changeInstance($pInstance)
// ---------------------------------------------------------------------
{
	$rc = OK;
	global $aConfiguration;
	$lHomeDir = $_ENV['HOME'];

	if ( ! in_array($pInstance, getInstances($aConfiguration)) ) {
		$rc = 501;
		error("Instance $pInstance does NOT exist in configuration file (rc=$rc).");
		return $rc;
	}

	print "\n";
	print "Changing instance $pInstance:\n";
	// Just to make sure it is set:
	if ( ! isset($aConfiguration[$pInstance]['name']) ) {
		$aConfiguration[$pInstance]['name'] = $pInstance;
	}


	// Ask for basedir

	$basedir = askForBasedir($aConfiguration[$pInstance]);
	$aConfiguration[$pInstance]['basedir'] = $basedir;

	// Do not create directory here otherwise link will fail later
	/*
	if ( ! is_dir($aConfiguration[$pInstance]['basedir']) ) {
		print "WARNING: Directory " . $aConfiguration[$pInstance]['basedir'] . " is NOT a directory or does not exist.\n";
		print "Creating directory " . $aConfiguration[$pInstance]['basedir'] . "\n";
		mkdir($aConfiguration[$pInstance]['basedir'], 0755, true);
	}
	*/


	// Install binaries as well

	if ( (is_dir($aConfiguration[$pInstance]['basedir']) && (count(glob($aConfiguration[$pInstance]['basedir'] . '/*')) == 0))
	    || ( ! file_exists($aConfiguration[$pInstance]['basedir']))
	   ) {

		print "\n";
		$key = answerQuestion("Shall I download and install mysql binaries under " . $aConfiguration[$pInstance]['basedir'] . " (Y/n)? ", array('y', 'n'), 'y');
		if ( $key == 'y' ) {

			$basedir = $aConfiguration[$pInstance]['basedir'];

			if ( $basedir == '' ) {
				$rc = 502;
				error("MyEnv is not installed yet. Please install MyEnv first with the command installMyEnv.php");
				exit($rc);
			}

			// require("$basedir/lib/myEnv.inc");
			// $lConfFile = $_ENV['MYENV_BASE'] . '/' . 'etc/' . 'myenv.conf';
			$lConfFile = '/etc/myenv/myenv.conf';
			debug("ConfFile: $lConfFile");

			if ( ! file_exists($lConfFile) ) {
				$rc = 503;
				error("MyEnv configuration file $lConfFile does not exist.");
				exit($rc);
			}

			list($ret, $aConfiguration) = getConfiguration($lConfFile);

			$lProductDir = $_ENV['HOME'] . '/' . 'product';
			$lTmpDir     = $_ENV['HOME'] . '/' . 'tmp';

			// Do not create otherwiese link below will fail
			/*
				if ( ! is_dir($lProductDir) ) {
				echo "\n";
				echo "INFO: Product directory $lProductDir does not exist yet.\n";
				echo "INFO: Creating it now.\n";
				mkdir($lProductDir, 0755, true);
			}
			*/

			if ( ! is_dir($lTmpDir) ) {
				echo "\n";
				echo "INFO: Tmp directory $lTmpDir does not exist yet.\n";
				echo "INFO: Creating it now.\n";
				mkdir($lTmpDir, 0755, true);
			}

			// Download URL

			print "\n";
			print "For example:\n";
			$aSources = array(
			  'http://dev.mysql.com/get/Downloads/MySQL-5.5/mysql-5.5.40-linux2.6-x86_64.tar.gz/from/http://cdn.mysql.com/'
			, 'http://dev.mysql.com/get/Downloads/MySQL-5.6/mysql-5.6.21-linux-glibc2.5-x86_64.tar.gz/from/http://cdn.mysql.com/'
			, 'http://www.percona.com/redir/downloads/Percona-Server-5.5/Percona-Server-5.5.39-36.0/binary/tarball/Percona-Server-5.5.39-rel36.0-697.Linux.x86_64.tar.gz'
			, 'http://www.percona.com/redir/downloads/Percona-Server-5.6/Percona-Server-5.6.20-68.0/binary/tarball/Percona-Server-5.6.20-rel68.0-656.Linux.x86_64.tar.gz'
			, 'https://downloads.mariadb.org/f/mariadb-5.5.39/bintar-centos5-amd64/mariadb-5.5.39-linux-x86_64.tar.gz/from/http%3A/ftp.ddg.lth.se/mariadb?server'
			, 'https://downloads.mariadb.org/f/mariadb-10.0.14/bintar-linux-x86_64/mariadb-10.0.14-linux-x86_64.tar.gz/from/http%3A/ftp.ddg.lth.se/mariadb?server'
			);
			foreach ( $aSources as $source ) {
				print '  ' . $source . "\n";
			}
			print "\n";
			do {
				$url = readline("Please enter the URL for the binary-tarball (Linux-Generic Compressed TAR) to download: ");
			} while ( $url == '');

			$parts = parse_url($url);

			echo "\n";
			$lVersion = extractVersion($parts['path']);
			if ( $lVersion == '' ) {
				echo "Version cannot be determined.\n";
			}
			else {
				echo "The version is: $lVersion\n";
			}
			// todo: compare download with platform

			// todo: check if wget or curl is available
			// downloadChunkedFile($url, $outfile)

			// download
			echo "\nPlease be patient, this can take a while...\n";
			$command = "wget --trust-server-names --directory-prefix=$lTmpDir $url 2>&1";
			echo $command . "\n";
			list($ret, $output, $stdout, $stderr) = my_exec($command);

			if ( $ret != 0 ) {
				$rc = 504;
				error("Download failed. Please try again manually (rc=$rc).");
				error(print_r($stderr, true));
			}

			// get binary path from stdout!
			// Saving to: `/home/myenv/tmp/mysql-5.1.41-linux-i686-glibc23.tar.gz'
			if ( ($rc == 0) && preg_match('|.*/(.*\.tar\.gz)|', $parts['path'], $matches) ) {
				$lTarBall = $matches[1];
				echo "Tar-ball file is: $lTarBall\n";
			}
			else {
				$rc = 505;
				error("Cannot extract filename from output. Please try manually (rc=$rc).");
				error("tar xf $lTmpDir" . '/' . 'mysql-5.6.21.tar.gz' . " --directory $lProductDir");
			}

			// todo: check checksum

			// extract binary
			if ( $rc == 0 ) {
				$command = "tar xf $lTmpDir" . '/' . "$lTarBall --directory $lProductDir";
				echo $command . "\n";
				list($ret, $output, $stdout, $stderr) = my_exec($command);

				$lBinaryDir = substr($lTarBall, 0, strlen($lTarBall)-strlen('.tar.gz'));

				// create link
				echo "ln -s $lProductDir" . '/' . $lBinaryDir . ' ' . $lProductDir . '/' . 'mysql-' . "$lVersion\n";
				symlink($lProductDir . '/' . $lBinaryDir, $lProductDir . '/' . 'mysql-' . $lVersion);
			}
		}
	}
	else {
		print "\n";
		print $aConfiguration[$pInstance]['basedir'] . " seems to exist and has already some files in it.\n";
		print "Omitting installation of binary tar ball.\n";
	}

	// datadir
	print "\n";
	print "Set datadir. The datadir is the directory where your MySQL data are located.\n";
	if ( isset($aConfiguration[$pInstance]['datadir']) && ($aConfiguration[$pInstance]['datadir'] != '') ) {
		$default = $aConfiguration[$pInstance]['datadir'];
	}
	else {
		$default = "$lHomeDir/data/$pInstance";
	}
	print "Example: /usr/local/mysql/data, /opt/mysql/data or /var/lib/mysql or " . $aConfiguration[$pInstance]['basedir'] . "/data or [$lHomeDir/data/$pInstance] or $default";
	print "\n";
	$aConfiguration[$pInstance]['datadir'] = rtrim(readline("datadir = "), '/');
	print "\n";

	if ( isset($aConfiguration[$pInstance]['datadir']) && ($aConfiguration[$pInstance]['datadir'] == '') ) {
		$aConfiguration[$pInstance]['datadir'] = $default;
	}
	if ( is_dir($aConfiguration[$pInstance]['datadir']) ) {
		print "WARNING: directory " . $aConfiguration[$pInstance]['datadir'] ." already exist.\n";
	}
	else {
		print "Creating directory " . $aConfiguration[$pInstance]['datadir'] . "\n";
		mkdir($aConfiguration[$pInstance]['datadir'], 0755, true);
	}


	// Install instance

	installInstance($aConfiguration[$pInstance]);


	// port

	print "\n";
	print "Set port.\n";
	if ( isset($aConfiguration[$pInstance]['port']) && ($aConfiguration[$pInstance]['port'] != '') ) {
		$default = $aConfiguration[$pInstance]['port'];
	}
	else {
		$default = 3306;
	}
	print "Example: 3307 or [$default]\n";
	$input = intval(readline("port    = "));
	if ( $input == 0 ) {
		$aConfiguration[$pInstance]['port'] = $default;
	}
	else {
		$aConfiguration[$pInstance]['port'] = $input;
	}
	print "\n";

	// Check if port already is in use
	foreach( getInstances($aConfiguration) as $instance ) {

		if ( ($aConfiguration[$instance]['port'] == $aConfiguration[$pInstance]['port']) && ($instance != $pInstance) ) {
			print "WARNING: Port " . $aConfiguration[$pInstance]['port'] . " already exists under instance $instance.\n";
		}
	}


	// Ask for socket

	$socket = askForSocket($aConfiguration[$pInstance]);
	$aConfiguration[$pInstance]['socket'] = $socket;


	// Ask for my.cnf

	$mycnf = askForMyCnf($aConfiguration[$pInstance]);
	$aConfiguration[$pInstance]['my.cnf'] = $mycnf;

	return $rc;
}

// ---------------------------------------------------------------------
function getInstances($aConfiguration)
// ---------------------------------------------------------------------
{
	$aInstance = array();

	foreach ( $aConfiguration as $key => $value ) {

		if ( $key != 'default' ) {
			array_push($aInstance, $key);
		}
	}
	return $aInstance;
}

// ---------------------------------------------------------------------
function installInstance($aConfiguration)
// ---------------------------------------------------------------------
{
	// Check if datadir exists and is empty directory
	if ( is_dir($aConfiguration['datadir']) && (count(glob($aConfiguration['datadir'] . '/mysql/*')) == 0) ) {

		print "\n";
		$key = answerQuestion("Shall I install a mysql database under " . $aConfiguration['datadir'] . " (Y/n)? ", array('y', 'n'), 'y');
		if ( $key == 'y' ) {

			if ( file_exists($aConfiguration['basedir'] . '/scripts/mysql_install_db') ) {
				$bin = $aConfiguration['basedir'] . '/scripts/mysql_install_db';
			}
			else {
				$bin = $aConfiguration['basedir'] . '/bin/mysql_install_db';
			}

			if ( file_exists($bin) && is_executable($bin) ) {
				print "Installing MySQL instance. Please be patient, this can take a while...\n";
				$cmd = $bin . " --datadir=" . $aConfiguration['datadir'] . " --basedir=" . $aConfiguration['basedir'];
				print "  $cmd\n";
				list($ret, $output, $stdout, $stderr) = my_exec($cmd);
				// var_dump($ret, $output, $stdout, $stderr);
				if ( $ret != OK ) {
					print "ERROR:\n";
					foreach ( $stdout as $line ) {
						print $line . "\n";
					}
					foreach ( $stderr as $line ) {
						print $line . "\n";
					}

					// Check if SELinux or AppArmor is installed

					print "Please check if advanced Linux security is enabled.\n";
					$aUname = posix_uname();
					if ( $aUname['sysname'] == 'Linux' ) {

						$cmd = 'lsb_release -s -i';
						list($ret, $distribution, $stdout, $stderr) = my_exec($cmd);
						if ( ($distribution == 'CentOS')
							|| ($distribution == 'RedHat')
							|| ($distribution == 'Fedora')
							) {
							print "\nINFO: SELinux is in status (sudo sestatus):\n";
							`/usr/sbin/sestatus | egrep 'status|mode'`;
							print "\n";
						}
						elseif ( ($distribution == 'Ubuntu')
									|| ($distribution == 'Debian')
									|| ($distribution == 'SUSE')
									|| ($distribution == 'openSUSE project')
									) {
							$cmd = '/usr/sbin/apparmor_status';
							if ( is_executable($cmd) ) {
								print "\nINFO: AppArmor is in status (sudo apparmor_status):\n";
								`$cmd`;
								print "\n";
							}
						}
						else {
							print "I do not know and support distribution: $distribution\n";
						}
					}

					if ( $distribution == 'openSUSE project' ) {
						print "  INFO: There is a bug in some SuSE installers (FATAL ERROR: Could not find /fill_help_tables.sql).\n";
					}
				}
			}
			else {
				print "ERROR: File $bin does not exists or is not executable.\n";
				print "Omitting installation of mysql database.\n";
			}
		}
	}
	else {
		print $aConfiguration['datadir'] . " seems to exist and has already some files in it.\n";
		print "Omitting installation of mysql database.\n";
	}
}

// ---------------------------------------------------------------------
function askForMyCnf($aConfiguration)
// ---------------------------------------------------------------------
{
	$aMyCnfCandidates = array(
		'/etc/my.cnf'
	, '/etc/mysql/my.cnf'
	, $aConfiguration['datadir'] . '/my-' . $aConfiguration['port'] . '.cnf'
	, $aConfiguration['datadir'] . '/my.cnf'
	);

	// Default my.cnf:
	if ( isset($aConfiguration['my.cnf']) && ($aConfiguration['my.cnf'] == '') ) {
		$default = $aConfiguration['datadir'] . "/my.cnf";
	}
	elseif ( isset($aConfiguration['my.cnf']) && ($aConfiguration['my.cnf'] != '') ) {
		$default = $aConfiguration['my.cnf'];
		array_push($aMyCnfCandidates, $default);
	}
	else {
		$default = $aConfiguration['datadir'] . '/my.cnf';
	}

	$aMyCnfCandidates = array_unique($aMyCnfCandidates);

	print "\n";
	print "Choose location of my.cnf:\n";
	print "Example: ";

	foreach ( $aMyCnfCandidates as $mycnf ) {
		if ( $mycnf == $default ) {
			print '[' . $mycnf . ']';
		}
		else {
			print $mycnf;
		}
		print ' ';
	}
	print "\n";

	$aConfiguration['my.cnf'] = readline("my.cnf  = ");
	if ( $aConfiguration['my.cnf'] == '' ) {
		$aConfiguration['my.cnf'] = $default;
	}
	print "\n";

	if ( $aConfiguration['my.cnf'] == '' ) {
		print "WARNING: empty my.cnf!\n";
		print "Please create manually.\n";
	}
	// Create my.cnf if not exits
	elseif ( ! is_file($aConfiguration['my.cnf']) ) {
		print "INFO: There is no such file " . $aConfiguration['my.cnf'] . " yet.\n";
		print "Creating file " . $aConfiguration['my.cnf'] . "\n";

		// todo: would be better to get it from outside!
		$basedir = dirname(dirname(__FILE__));
		// read our template
		$tpl = file_get_contents($basedir . '/etc/my.cnf.template');
		// Substitue all %VARIABLES%
		$aSearch = array('%MYCNF%', '%PORT%', '%SOCKET%', '%BASEDIR%', '%DATADIR%', '%SERVERID%');
		$aReplace = array($aConfiguration['my.cnf'], $aConfiguration['port'], $aConfiguration['socket'], $aConfiguration['basedir'], $aConfiguration['datadir'], $aConfiguration['port']);
		$cnf = str_replace($aSearch, $aReplace, $tpl);

		ini_set('track_errors', 1);
		if ( $fh = @fopen($aConfiguration['my.cnf'], 'w') ) {
			fwrite($fh, $cnf);
			fclose($fh);
			chmod($aConfiguration['my.cnf'], 0600);
		}
		// Opening my.cnf for writing failed (e.g. /etc/my.cnf)
		else {
			$rc = 506;
			error("Writing to file " . $aConfiguration['my.cnf'] . " failed (rc=$rc)");
			error($php_errormsg);
		}
	}

	print "\n";

	return $aConfiguration['my.cnf'];
}

// ---------------------------------------------------------------------
function askForSocket($aConfiguration)
// ---------------------------------------------------------------------
{
	$aSocketCandidates = array(
		'/tmp/mysql-' . $aConfiguration['port'] . '.sock'
	, '/tmp/mysql-' . $aConfiguration['name'] . '.sock'
	, '/tmp/mysql.sock'
	, '/var/run/mysqld/mysql.sock'
	);

	// Default socket:
	if ( isset($aConfiguration['socket']) && ($aConfiguration['socket'] == '') ) {
		// /tmp/mysql-3306.sock
		$default =  "/tmp/mysql-" . $aConfiguration['port'] . ".sock";
	}
	elseif ( isset($aConfiguration['socket']) && ($aConfiguration['socket'] != '') ) {
		$default =  $aConfiguration['socket'];
		array_push($aSocketCandidates, $default);
	}
	else {
		$default = '/tmp/mysql.sock';
	}

	$aSocketCandidates = array_unique($aSocketCandidates);

	print "\n";
	print "Set socket.\n";
	print "Example: ";
	foreach ( $aSocketCandidates as $socket ) {
		if ( $socket == $default ) {
			print '[' . $socket . ']';
		}
		else {
			print $socket;
		}
		print ' ';
	}
	print "\n";
	$aConfiguration['socket'] = readline("socket  = ");
	if ( $aConfiguration['socket'] == '' ) {
		$aConfiguration['socket'] = $default;
	}
	print "\n";

	return $aConfiguration['socket'];
}

// ---------------------------------------------------------------------
function askForBasedir($aConfiguration)
// ---------------------------------------------------------------------
{
	$lHomeDir = $_ENV['HOME'];

	// Guess candidates from /home/mysql/product
	$aCandidates = glob($lHomeDir . '/product/*', GLOB_ONLYDIR);

	$aBasedirCandidates = array_merge($aCandidates, array(
		'/usr/local/mysql'
	, '/opt/mysql'
	//  , $lHomeDir . '/product/mysql-5.6.13'
	, '/usr'
	));

	// Default basedir:
	if ( isset($aConfiguration['basedir']) && ($aConfiguration['basedir'] == '') ) {
		// /home/mysql/product/mysql
		$default =  $lHomeDir . '/product/mysql';
	}
	elseif ( isset($aConfiguration['basedir']) && ($aConfiguration['basedir'] != '') ) {
		$default =  $aConfiguration['basedir'];
		array_push($aBasedirCandidates, $default);
	}
	else {
		$default = '/usr';
	}

	$aBasedirCandidates = array_unique($aBasedirCandidates);

	print "\n";
	print "Set basedir. The basedir is the directory where your MySQL binaries are located.\n";
	print "Example: \n";
	foreach ( $aBasedirCandidates as $basedir ) {
		if ( $basedir == $default ) {
			print '[' . $basedir . ']';
		}
		else {
			print $basedir;
		}
		print "\n";
	}
	print "\n";

	$aConfiguration['basedir'] = rtrim(readline("basedir = "), '/');
	if ( $aConfiguration['basedir'] == '' ) {
		$aConfiguration['basedir'] = $default;
	}
	print "\n";

	return $aConfiguration['basedir'];
}

// ---------------------------------------------------------------------
function downloadChunkedFile($url, $outfile)
// ---------------------------------------------------------------------
{
	$chunksize = 10 * (1024 * 1024);

	$parts = parse_url($url);
	$i_handle = fsockopen($parts['host'], 80, $errstr, $errcode, 5);
	$o_handle = fopen($outfile, 'wb');

	if ( ($i_handle === false) || ($o_handle === false) ) {
		return false;
	}

	if ( ! empty($parts['query']) ) {
		$parts['path'] .= '?' . $parts['query'];
	}


  // Send the request to the server for the file

	$request  = 'GET {' . $parts['path'] . "} HTTP/1.1\r\n";
	$request .= "Host: {" . $parts['host'] . "}\r\n";
	$request .= "User-Agent: Mozilla/5.0\r\n";
	$request .= "Keep-Alive: 115\r\n";
	$request .= "Connection: keep-alive\r\n\r\n";
	fwrite($i_handle, $request);


	// Now read the headers from the remote server. We'll need
	// to get the content length.

	$headers = array();
	while ( ! feof($i_handle) ) {
		$line = fgets($i_handle);
		if ($line == "\r\n") {
			break;
		}
		$headers[] = $line;
	}


	// Look for the Content-Length header, and get the size
	// of the remote file.

	$length = 0;
	foreach ( $headers as $header ) {
		if ( stripos($header, 'Content-Length:') == 0 ) {
			$length = (int)str_replace('Content-Length: ', '', $header);
			break;
		}
	}


	// Start reading in the remote file, and writing it to the
	// local file one chunk at a time.

	$cnt = 0;
	while ( ! feof($i_handle) ) {
		$buf = '';
		$buf = fread($i_handle, $chunksize);
		$bytes = fwrite($o_handle, $buf);
		if ( $bytes === false ) {
			return false;
		}
		$cnt += $bytes;


		// We're done reading when we've reached the conent length

		if ( $cnt >= $length ) {
			break;
		}
	}

	fclose($i_handle);
	fclose($o_handle);
	return $cnt;
}

?>
